@@ -27,7 +27,9 @@ import urllib.parse as up
 import random, string, json, uuid, re
 import logging
 
-from qsAPI import __version__
+__version__ = "1.16"
+__updated__ = '20/01/2019'
+
 
 
 class _Controller(object):
@@ -200,6 +202,7 @@ class _Controller(object):
             self.response = self.session.send(self.response.next, verify=self._verify, allow_redirects=False)
             
         self.log.debug('RECV: %s',self.response.text)
+        self.response.raise_for_status()
         
         return(self.response)
 
@@ -436,21 +439,12 @@ class QRS(object):
         return(r)
     
 
-
-    def ping(self):
-        '''
-        @return: "Ping successful", if there are no problems contacting the Qlik Sense Repository Service (QRS).
-        '''
-        return self.driver.call('GET', '/qrs/ssl/ping')
-
-
-
         
     def getServerVersion(self):
         '''
         @Function: retrieve the server version
         '''
-        return Version(self.driver.call('GET', '/qrs/about').json().get('buildVersion'))
+        return Version(self.driver.get('/qrs/about').json().get('buildVersion'))
 
  
  
@@ -503,17 +497,16 @@ class QRS(object):
 
     
     
-    def AppDictAttributes(self, guid='full', key='name', attr='id'):
+    def AppDictAttributes(self, puid='full', pFilter=None, key='name', attr='id'):
         '''@Function: retrieve a mapping of apps attributes
-           @param pId: limmit the scope to the App {GUID}
+           @param pId: limmit the scope to the App {UUID}
+           @param pFilter: filter the entities before calculating the number of entities.
            @param key: the attribute to be the key
            @param attr: the attribute value to retrieve (single value or list)
            @return: dict(key:attr)
         '''
-        
-        apipath='/qrs/app/{guid}'.format(guid=guid)
-        
-        return self._toDict(self.driver.get(apipath), guid, key, attr)    
+        apipath='/qrs/app/{puid}'.format(puid=puid)
+        return self._toDict(self.driver.get(apipath, param={'filter':pFilter}), puid, key, attr)    
         
     
 
@@ -528,11 +521,12 @@ class QRS(object):
 
 
     
-    def AppExport(self, pId, filename=None, skipdata='true'):
+    def AppExport(self, pId, filename=None, skipData=False):
         '''
         @Function: Get an export qvf for an existing app, identified by {id}.
         @param pId: app GUI
         @param filename: target path filename
+        @param skipData: if True App will be emptied of data
         @return : stored application
         '''
         file= filename if filename else pId+'.qvf'
@@ -545,7 +539,8 @@ class QRS(object):
             return(r)
         
         #Current API method
-        r=self.driver.post('/qrs/app/{id}/export/{token}?skipData={skipdata}'.format(id=pId, token=uuid.uuid4(), skipdata=skipdata))
+        param={'skipData':skipData}
+        r=self.driver.post('/qrs/app/{id}/export/{token}'.format(id=pId, token=uuid.uuid4()), param)
         if r.ok:
             r=self.driver.download(r.json()['downloadPath'], file)
         return(r)
@@ -568,7 +563,7 @@ class QRS(object):
     def AppGet(self, pId='full', pFilter=None):
         '''
         @Function: retrieve App information
-        @param pId: App GUID 
+        @param pId: App UUID 
         @param pFilter: filter the entities before calculating the number of entities. 
         @return : json response
         '''
@@ -609,7 +604,7 @@ class QRS(object):
     def AppUpdate(self, pId, pData):
         '''
         @Function: update App info referenced 
-        @param pId: App GUID 
+        @param pId: App UUID 
         '''
         return self.driver.put('/qrs/app/{id}'.format(id=pId), data=pData)
     
@@ -617,8 +612,8 @@ class QRS(object):
     def AppReplace(self, pId, pAppId):
         '''
         @Function: Replace an app, identified by {appid}, with the app identified by {id}. 
-        @param pId: App GUID 
-        @param pAppId: target App GUID
+        @param pId: source App UUID 
+        @param pAppId: target App UUID
 
         If the replaced app is published, only the sheets that were originally published with the app are replaced.
         If the replaced app is not published, the entire app is replaced.
@@ -630,7 +625,7 @@ class QRS(object):
     def AppDelete(self, pId):
         '''
         @Function: delete App referenced 
-        @param pId: App GUID 
+        @param pId: App UUID 
         '''
         return self.driver.delete('/qrs/app/{id}'.format(id=pId))
     
@@ -638,6 +633,63 @@ class QRS(object):
     #=========================================================================================
     
     
+    def AppObjectGet(self, pId='full', pFilter=None):
+        '''
+        @Function: retrieve AppObject information
+        @param pId: AppObject UUID 
+        @param pFilter: filter the entities before calculating the number of entities. 
+        @return : json response
+        '''
+        return self.driver.get('/qrs/app/object/{id}'.format(id=pId), param={'filter':pFilter}).json()
+    
+    
+    def AppObjectCount(self, pFilter=None):
+        '''
+        @Function: retrieve AppObject count information
+        @param pFilter: filter the entities before calculating the number of entities. 
+        @return : json response
+        '''
+        return self.driver.get('/qrs/app/object/count', param={'filter':pFilter}).json()
+    
+    
+    def AppObjectUpdate(self, pId, pData):
+        '''
+        @Function: retrieve AppObject information
+        @param pId: AppObject UUID  
+        @param pData: AppObject attributes
+        '''
+        return self.driver.put('/qrs/app/object/{id}'.format(id=pId), data=pData)
+    
+    
+    def AppObjectApprove(self, pId, pApprove=True):
+        '''
+        @Function: Set AppObject approve status
+        @param pId: AppObject UUID  
+        @param pApprove: True / False
+        '''
+        return self.driver.post('/qrs/app/object/{id}/{status}'.format(id=pId, status='approve' if pApprove else 'unapprove'))
+    
+    
+    def AppObjectPublish(self, pId, pPublish=True):
+        '''
+        @Function: Set AppObject publish status
+        @param pId: AppObject UUID  
+        @param pPublish: True / False
+        '''
+        return self.driver.put('/qrs/app/object/{id}/{status}'.format(id=pId, status='publish' if pPublish else 'unpublish'))
+    
+    
+    def AppObjectDelete(self, pId):
+        '''
+        @Function: Delete AppObject
+        @param pId: AppObject UUID  
+        '''
+        return self.driver.delete('/qrs/app/object/{id}'.format(id=pId))
+    
+        
+    #=========================================================================================
+    
+    
     def StreamCreate(self, pName, pProperties=[] , pTags=[], pUUID=None):
         '''
         @Function: create a Stream
@@ -661,7 +713,7 @@ class QRS(object):
     def StreamGet(self, pId='full', pFilter=None):
         '''
         @Function: retrieve Stream information
-        @param pId: Stream GUID 
+        @param pId: Stream UUID 
         @param pFilter: filter the entities before calculating the number of entities. 
         @return : json response
         '''
@@ -672,7 +724,8 @@ class QRS(object):
     def StreamUpdate(self, pId, pData):
         '''
         @Function: update Stream info referenced 
-        @param pId: Stream GUID 
+        @param pId: Stream UUID 
+        @param pData: stream attributes
         '''
         return self.driver.put('/qrs/stream/{id}'.format(id=pId), data=pData)
     
@@ -681,22 +734,23 @@ class QRS(object):
     def StreamDelete(self, pId):
         '''
         @Function: delete Stream referenced 
-        @param pId: Stream GUID 
+        @param pId: Stream UUID 
         @return : json response
         '''
         return self.driver.delete('/qrs/stream/{id}'.format(id=pId))
     
     
     
-    def StreamDictAttributes(self, pStreamID='full', key='name', attr='id'):
+    def StreamDictAttributes(self, pStreamID='full', pFilter=None, key='name', attr='id'):
         '''@Function: retrieve a mapping of Stream attributes
            @param pStreamID: limmit the scope to the Stream {UID}
+		   @param pFilter: filter the entities before calculating the number of entities.
            @param key: the attribute to be the key
            @param attr: the attribute value to retrieve (single value or list)
            @return: dict(key:attr)
         '''
         apipath='/qrs/stream/{uid}'.format(uid=pStreamID)            
-        return self._toDict(self.driver.get(apipath), pStreamID, key, attr) 
+        return self._toDict(self.driver.get(apipath, param={'filter':pFilter}), pStreamID, key, attr) 
     
     
     #=========================================================================================     
@@ -732,83 +786,29 @@ class QRS(object):
         return self.driver.delete('/qrs/user/{id}'.format(id=pUserID))
     
     
-    def UserDictAttributes(self, pUserID='full', key='name', attr='id'):
+    def UserDictAttributes(self, pUserID='full', pFilter=None, key='name', attr='id'):
         '''@Function: retrieve a mapping of user attributes
            @param pUserID: limmit the scope to the User {UID}
+		   @param pFilter: filter the entities before calculating the number of entities.
            @param key: the attribute to be the key
            @param attr: the attribute value to retrieve (single value or list)
            @return: dict(key:attr)
         '''
         apipath='/qrs/user/{uid}'.format(uid=pUserID)            
-        return self._toDict(self.driver.get(apipath),pUserID,key,attr)
+        return self._toDict(self.driver.get(apipath, param={'filter':pFilter}),pUserID,key,attr)
     
     
     #=========================================================================================
-
-    def TaskGet(self, pFilter=None):
+    
+    def SystemRulesCreate(self, param):
         '''
-        @Function: retrieve Task information
-        @param pFilter: filter the entities
+        @Function: create a SystemRule
         @return : json response
-        '''
-        return self.driver.get('/qrs/task/full', param={'filter': pFilter}).json()
-
-    def TaskStart(self, taskid):
-        '''
-        @Function: Starts a task by id and waits until a slave starts to execute a task
-        @param taskid: taskid of the task to start
-        '''
-        return self.driver.post('/qrs/task/{taskid}/start'.format(taskid=taskid))
-
-    def TaskStartSynchronous(self, taskid):
-        '''
-        @Function: Starts a task by id and waits until a slave starts to execute a task
-        @param taskid: taskid of the task to start
-        '''
-        return self.driver.post('/qrs/task/{taskid}/start/synchronous'.format(taskid=taskid))
-
-
-    def TaskStartByName(self, taskname):
-        '''
-        @Function: Starts a task by name
-        @param taskname: Name of the task to start
-        '''
-        return self.driver.post('/qrs/task/start', param={'name': taskname})
-
-    def TaskStartMany(self, taskids):
-        '''
-        @Function: Starts multiple tasks
-        @param taskids: list of id's of the task to start
-            Sample list: ["6ca1c5f2-2742-44d5-8adf-d6cba3701a4e","965ca0cf-952f-4502-a65e-2a82e3de4803"]
-        '''
-        return self.driver.post('/qrs/task/start/many', data=taskids)
-
-    def TaskStartByNameSynchronous(self, taskname):
-        '''
-        @Function: Starts a task and waits until a slave starts to execute a task
-        @param taskname: Name of the task to start
-        '''
-        return self.driver.post('/qrs/task/start/synchronous', param={'name': taskname})
-
-    def TaskStop(self, taskid):
-        '''
-        @Function: Stops a task
-        @param taskid: id of the task to stop
-        '''
-        return self.driver.post('/qrs/task/{taskid}/stop'.format(taskid=taskid))
-
-    def TaskStopMany(self, taskids):
-        '''
-        @Function: Stops multiple tasks
-        @param taskname: list of id's of the task to stop
-            Sample list: ["6ca1c5f2-2742-44d5-8adf-d6cba3701a4e","965ca0cf-952f-4502-a65e-2a82e3de4803"]
-        '''
-        return self.driver.post('/qrs/task/stop/many', data=taskids)
-
-    #=========================================================================================
-        
+        ''' 
+        return self.driver.post('/qrs/systemrule', data=param).json()
+    
    
-    def SystemRules(self, pFilter=None):
+    def SystemRulesGet(self, pFilter=None):
         '''
         @Function: Get the system rules
         '''
@@ -816,21 +816,36 @@ class QRS(object):
    
    
     
-    def SystemRulesDictAttributes(self, pRuleID='full', key='name', attr='id'):
+    def SystemRulesDictAttributes(self, pRuleID='full', pFilter=None, key='name', attr='id'):
         '''@Function: retrieve a mapping of rules attributes
            @param pRuleID: limmit the scope to the Rule {UID}
+		   @param pFilter: filter the entities before calculating the number of entities.
            @param key: the attribute to be the key
            @param attr: the attribute value to retrieve (single value or list)
            @return: dict(key:attr)
         '''
         apipath='/qrs/systemrule/{uid}'.format(uid=pRuleID)            
-        return self._toDict(self.driver.get(apipath),pRuleID,key,attr)
-    
+        return self._toDict(self.driver.get(apipath, param={'filter':pFilter}),pRuleID,key,attr)
     
     
     #=========================================================================================
     
     
+    
+    def ReloadTaskGet(self, pId='full', pFilter=None):
+        '''
+        @Function: retrieve ReloadTask information
+        @param pId: ReloadTask UID 
+        @param pFilter: filter the entities before calculating the number of entities. 
+        @return : json response
+        '''
+        return self.driver.get('/qrs/reloadtask/{id}'.format(id=pId), param={'filter':pFilter}).json()
+    
+    
+    
+    #=========================================================================================
+     
+    
     def PropertiesGet(self, pFilter=None):
         '''
         @Function: Get the system rules
@@ -841,3 +856,106 @@ class QRS(object):
     #=========================================================================================
     
     
+    def TagsDictAttributes(self, pTagID='full', pFilter=None, key='name', attr='id'):
+        '''@Function: retrieve a mapping of tags attributes
+           @param pRuleID: limmit the scope to the Tag {UID}
+		   @param pFilter: filter the entities before calculating the number of entities.
+           @param key: the attribute to be the key
+           @param attr: the attribute value to retrieve (single value or list)
+           @return: dict(key:attr)
+        '''
+        apipath='/qrs/tag/{uid}'.format(uid=pTagID)            
+        return self._toDict(self.driver.get(apipath, param={'filter':pFilter}),pTagID,key,attr)
+
+    
+    #=========================================================================================
+    
+    
+    class LicenseType:
+        UserAccess='useraccesstype'
+        LoginAccess='loginaccesstype'
+        ProfessionalAccess='professionalaccesstype'
+        AnalyzerAccess='analyzeraccesstype'
+    
+    
+    def LicenseUsageSummary(self):
+        '''
+        @Function: Get the license summary
+        '''
+        return self.driver.get('qrs/license/accesstypeinfo').json()
+    
+    
+    def LicenseAccessGet(self, licenseType):
+        '''
+        @Function: Get a user access licenses
+        @param licenseType: LicenseType***Access enumeration
+        '''
+        return self.driver.get('qrs/license/{}/full'.format(licenseType)).json()
+    
+    
+    def LicenseAccessDelete(self, licenseType, pLicID):
+        '''
+        @Function: Delete a user access license
+        @param licenseType: LicenseType***Access enumeration
+        @param pLicID: key of license
+        '''
+        return self.driver.delete('qrs/license/{}/{}'.format(licenseType, pLicID))
+    
+    
+    def LicenseAccessCount(self, licenseType):
+        '''
+        @Function: Retrieve the number of assigned access license
+        @param licenseType: LicenseType***Access enumeration
+        @param pLicID: key of licens
+        '''
+        return self.driver.get('/qrs/license/{}/count'.format(licenseType)).json()['value']
+
+
+    #=========================================================================================
+    
+
+if __name__ == "__main__":
+    '''
+    Alternative command line invocation, examples:
+        python qsAPI.py -s myServer -c dir/client.pem -Q QRS AppDictAttributes
+        python qsAPI.py -s myServer -c dir/client.pem -Q QRS -v INFO AppExport d8b120d7-a6e4-42ff-90b2-2ac6a3d92233 
+        python qsAPI.py -s myServer -c dir/client.pem -Q QRS -v INFO AppReload d8b120d7-a6e4-42ff-90b2-2ac6a3d92233
+        
+    '''
+    from argparse import ArgumentParser
+    import inspect
+    from pprint import pprint
+    
+    parser = ArgumentParser(description='qsAPI for QlikSense')
+    parser.add_argument('-s', dest='server', required=True, help='server hostname | hostname:port | https://hostname:port')
+    parser.add_argument('-u', dest='user', required=False, default='internal\\sa_repository', help='user in domain\\userid format.')
+    parser.add_argument('-p', dest='password', required=False, default=None, help='password credential (NTLM)')
+    parser.add_argument('-c', dest='certificate', required=False, help='path to client.pem certificate.')
+    parser.add_argument('-P', dest='vproxy', required=False, help='virtual proxy preffix if needed.')
+    parser.add_argument("-Q", dest="api", choices=['QPS','QRS'], default='QRS', required=True, help="service API")
+    parser.add_argument("-v", dest="verbose", choices=['DEBUG','INFO','WARNING','ERROR','CRITICAL'], default='INFO', help="set verbosity level")
+    parser.add_argument('--version', action='version', version='tools {}'.format(__version__))
+    parser.add_argument(dest='method', nargs='+', help='API method to call and arguments')
+    
+    # Process arguments
+    args = parser.parse_args()
+    Q=QPS if args.api == 'QPS' else QRS
+
+    if not (bool(args.password) != bool(args.certificate)):
+        print('ERROR: One and only one authentication method must be provided (password or certificate)')
+        sys.exit(-1)
+        
+    user=args.user.replace('\\\\','\\').split('\\')
+    user.append(args.password)
+    qr=Q(proxy=args.server, vproxy=args.vproxy, certificate=args.certificate, user=user, verbosity=args.verbose)
+    m=[x for x,y in inspect.getmembers(Q) if not x.startswith('_') ]
+    
+    cmd=args.method[0]
+    if cmd not in m:
+        print('ERROR: "{}" is not a method of {}, expected=> {}'.format(cmd, args.api, m))
+        sys.exit(-1)
+    
+    pprint(getattr(qr, cmd)(*args.method[1:]))
+    sys.exit(0)
+
+    
